
#ifndef DIHEDRALPOTENTIAL_H
#define DIHEDRALPOTENTIAL_H

#include "rumd/Potential.h"

class MoleculeData; class Sample;

///////////////////////////////////////////////////////
// Abstract Potential class
///////////////////////////////////////////////////////

class DihedralPotential : public Potential{
 public:
  MoleculeData *M;
  Sample *S;

  int num_blocks;
  int num_threads;
 
};



// Ryckaert-Bellemans potential

class DihedralRyckaert : public DihedralPotential{
 private:
  DihedralRyckaert(const DihedralRyckaert&);
  DihedralRyckaert& operator=(const DihedralRyckaert&);
  
 public:
  DihedralRyckaert(MoleculeData *, Sample *);
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};

///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////
template <int energy, class Simbox> 
__global__ void Ryckaert( float4 *r, float4 *f,  Simbox *simbox, float *simboxpointer, 
			  uint4 *dlist, uint1 *dtype, float *plist, 
			  float *d_epot_dihedral, float *d_dihedrals,
			  int num_threads, int num_dihedrals );


__global__ void DihedralResetForces( float4 *f, float4 *w );
__device__ inline void DihedralAtomicFloatAdd(float *address, float val);


#endif
